/*******************************************************************************
 *	BoapMc.h	BoapMc RPC protocol
 *			T.Barnaby,	BEAM Ltd,	2012-11-14
 *******************************************************************************
 */
#ifndef	BoapMc_HH
#define	BoapMc_HH

#include <BTypes.h>
#include <BComms.h>

// Main BOAP Types
enum BoapMcType		{ BoapMcTypeRpc=0, BoapMcTypeRpcReply=1, BoapMcTypeSignal=2 };

// BoapMc packet header
struct BoapMcPacketHead {
	BUInt8		length;
	BUInt8		addressTo;
	BUInt8		addressFrom;
	BUInt8		cmd;
	BUInt16		checksum;
};

// BoapMc packet
class BoapMcPacket {
public:
	BoapMcPacketHead	head;
	char			data[256 - sizeof(BoapMcPacketHead)];
};

/*******************************************************************************
 *	Base for all Client Objects
 *******************************************************************************
 */
class BoapMcClientObject {
public:
			BoapMcClientObject(BComms& comms);
	virtual		~BoapMcClientObject();
	void		setAddress(BUInt8 addressTo, BUInt8 addressFrom);

protected:
	BError		performCall();				///< Performs a RPC call to the named service
	BError		performSend();				///< Performs a send to the named service
	BError		performRecv();				///< Performs a receive

	BUInt32		oapiVersion;
	BComms&		ocomms;
	BUInt8		oaddressTo;
	BUInt8		oaddressFrom;
	BoapMcPacket	opacket;
};

class BoapMcSignalObject {
public:
			BoapMcSignalObject(BComms& comms);

protected:
	BError		performSend(BoapMcPacket& tx);			// Performs a send to the named service
	BComms&		ocomms;
};


/*******************************************************************************
 *	Main Server communications class
 *******************************************************************************
 */
class BoapMcServiceObject {
public:
			BoapMcServiceObject();
	virtual		~BoapMcServiceObject();

	virtual BError	process(BoapMcPacket& rx, BoapMcPacket& tx);
	virtual BError	processEvent(BoapMcPacket& rx);
protected:
	BError		sendEvent(BoapMcPacket& tx);
	BUInt32		oapiVersion;
};
#endif
